#include "analisadorpda.h"

#include <cv.h>
#include <cstdlib>
#include "contrast.h"
#include "produto.h"
#include "definitions.h"

AnalisadorPDA::AnalisadorPDA() : Analisador ()
{
    this->contraste = NULL;
    /* cria os dois classificadores */
    this->class_nomes = treina_classificador("./treinamento_imagens/treinamento_ordenado.csv");
    contr_ultimo_recorte = NULL;
}

void AnalisadorPDA::detecta_linhas(IplImage *pagina)
{
    IplImage *cinza = cvCloneImage(pagina);
    cvNot(cinza, cinza);
    this->linhas = cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 1);
    cvZero(this->linhas);

    /* parâmetros para o tamanho esperado de uma linha */
    int min_size = 390, line_thickness = 17;

    /* Linhas horizontais: */
    IplImage *hor = cvCloneImage(cinza);
    cvMorphologyEx(cinza, hor, NULL,
        cvCreateStructuringElementEx(1, line_thickness, 0, line_thickness/2, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvDilate(hor, hor, NULL, 2);
    cvMorphologyEx(hor, hor, NULL,
        cvCreateStructuringElementEx(min_size, 1, min_size/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);

    cvMorphologyEx(hor, hor, NULL,
                     cvCreateStructuringElementEx(500, 1, 250, 0, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);

    /* Linhas verticais: */
    IplImage *vert = cvCloneImage(cinza);
    cvMorphologyEx(cinza, vert, NULL,
        cvCreateStructuringElementEx(line_thickness, 1, line_thickness/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvDilate(vert, vert, NULL, 2);
    cvMorphologyEx(vert, vert, NULL,
        cvCreateStructuringElementEx(1, min_size, 0, min_size/2, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cvMorphologyEx(vert, vert, NULL,
                     cvCreateStructuringElementEx(1, 300, 0, 150, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);

    cvOr(vert, hor, this->linhas);
    cvReleaseImage(&hor);
    cvReleaseImage(&vert);
    cvDilate(this->linhas, this->linhas, NULL, 10);
//    cvSaveImage("linhas-noth.png", this->linhas, 0);
    cvThreshold(this->linhas, this->linhas, 30, 255, cv::THRESH_BINARY);
//    cvSaveImage("linhas.png", this->linhas, 0);
}


IplImage *AnalisadorPDA::segmenta_fundo(IplImage *pagina) {
//    cvSaveImage("original.png", pagina, 0);

    processa_usando_piramide(pagina, &(this->contraste), 3);
//    cvSaveImage("contr.png", contraste, 0);
    IplImage *fundo = cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 1);
    cvThreshold(this->contraste, fundo, 65, 255, cv::THRESH_BINARY);
    this->detecta_linhas(this->contraste);
    /* Tira ruido */
    cvMorphologyEx(fundo, fundo, NULL, NULL, cv::MORPH_OPEN, 1);
    /* Tira linhas */
    cvMorphologyEx(fundo, fundo, NULL,
                   cvCreateStructuringElementEx(17, 1, 8, 0, cv::MORPH_RECT, NULL), cv::MORPH_OPEN, 1);
    cvMorphologyEx(fundo, fundo, NULL,
                   cvCreateStructuringElementEx(1, 17, 0, 8, cv::MORPH_RECT, NULL), cv::MORPH_OPEN, 1);
//    cvSaveImage("fundo.png", fundo, 0);
    return fundo;
}

CvContour *AnalisadorPDA::encontra_produtos(IplImage *pagina, IplImage *fundo)
{
    CvContour *cc = (CvContour *) connected_components(fundo);
    cc = pre_processa_cc(cc, 2.0, 2.0, BIGGER);
    return cc;
}

CvRect AnalisadorPDA::recorte_rect(IplImage *pagina, CvContour *produto)
{
    CvRect r;
    IplImage *mask = cvCreateImage(cvSize(pagina->width + 2, pagina->height + 2), IPL_DEPTH_8U, 1);
    cvZero(mask);
    CvConnectedComp cc;
    CvPoint seed; seed.x = produto->rect.x; seed.y = produto->rect.y;
    cvFloodFill(this->linhas, seed, cvScalar(255), cvScalar(0), cvScalar(0), &cc, 4 | CV_FLOODFILL_MASK_ONLY, mask);
    cvReleaseImage(&mask);
    return cc.rect;
}

IplImage *AnalisadorPDA::mascara_de_componentes_conexas_conforme_classe(IplImage *recorte, IplImage *recorte_fundo, CvContour *produto, int classe_desejada, int usa_anterior)
{
    IplImage *contraste_recorte;
    IplImage *mask = cvCreateImage(cvGetSize(recorte), IPL_DEPTH_8U, 1);
    cvZero(mask);
    IplImage *th = cvCreateImage(cvGetSize(recorte), IPL_DEPTH_8U, 1);
    CvRect regiao_cc;
    if (usa_anterior) {
        contraste_recorte = contr_ultimo_recorte;
    } else {
        if (contr_ultimo_recorte)
            cvReleaseImage(&contr_ultimo_recorte);
        processa_usando_piramide(recorte, &contraste_recorte, 3);
    }
    contr_ultimo_recorte = contraste_recorte;
    pre_processa_imagem_para_classificacao(contraste_recorte, th, PRODUTOS_BLOCK, PRODUTOS_DELTA);
    cvDrawContours(th, (CvSeq *) produto, CV_RGB(0, 0, 0), CV_RGB(0, 0, 0), -1, CV_FILLED, 8,
        cvPoint(0, 0));
    CvContour *cc = (CvContour *) connected_components(th);
    float caracteristicas[8];
    for (CvContour *t = cc; t != NULL; t = (CvContour *) t->h_next) {
        regiao_cc.x = t->rect.x;
        regiao_cc.y = t->rect.y;
        regiao_cc.width = t->rect.width;
        regiao_cc.height = t->rect.height;
        extrai_caracteristicas(contraste_recorte, th, &regiao_cc, caracteristicas);
        /* classifica como nome de produto ou qq outra coisa */
        if (classifica(this->class_nomes, caracteristicas, 8) == classe_desejada) {
            cvRectangle(mask, cvPoint(t->rect.x, t->rect.y),
                cvPoint(t->rect.width + t->rect.x - 1, t->rect.height + t->rect.y - 1),
                CV_RGB(255, 255, 255), CV_FILLED, 8, 0);
        }
    }
    cvReleaseMemStorage(&cc->storage);
    cvReleaseImage(&th);
    return mask;
}


IplImage *AnalisadorPDA::encontra_nomes_de_produtos(IplImage *recorte, IplImage *recorte_fundo, CvContour *produto)
{
    IplImage *mask = mascara_de_componentes_conexas_conforme_classe(recorte, recorte_fundo, produto, CLASS_NOME, 0);
    cvMorphologyEx(mask, mask, NULL, cvCreateStructuringElementEx(15, 15, 0, 7, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);
    cvDilate(mask, mask, cvCreateStructuringElementEx(25, 25, 12, 12, CV_SHAPE_RECT, NULL), 1);
    return mask;
}

IplImage *AnalisadorPDA::encontra_precos(IplImage *recorte, IplImage *recorte_fundo, CvContour *produto)
{
    IplImage *mask = mascara_de_componentes_conexas_conforme_classe(recorte, recorte_fundo, produto, CLASS_PRECO, 1);
    cvMorphologyEx(mask, mask, NULL, cvCreateStructuringElementEx(15, 15, 0, 7, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);
    cvDilate(mask, mask, cvCreateStructuringElementEx(25, 25, 12, 12, CV_SHAPE_RECT, NULL), 1);
    return mask;
}

