#include "classifier.hpp"
#include <cstdlib>
#include <cstdio>
#include "definitions.h"

using namespace std;

CvKNearest *treina_classificador (char *caminho_do_arquivo)
{
    FILE *f;
    float viz1, viz2, p, asp, media, dp, w, h;
    int classe, i, n;
    float c, k, fll[8];
    CvMat *train_data, *classes;
    CvMat *teste;
    CvKNearest *knn;
    f = fopen(caminho_do_arquivo, "r");
    if (!f) {
        return NULL;    
    }
    fscanf(f, "%d\n", &n);
    train_data = cvCreateMat(n, 8, CV_32FC1);
    classes = cvCreateMat(n, 1, CV_32FC1);
    teste = cvCreateMat(1, 8, CV_32FC1);
    for (i = 0; i < n; i++) {
        fscanf(f, "%d;%f;%f;%f;%f;%f;%f;%f;%f\n", &classe, &viz1, &viz2, &p, &asp, &media, &dp, &w, &h);
        cvSet2D(classes, i, 0, cvScalar(classe));
        cvSet2D(train_data, i, 0, cvScalar(viz1));
        cvSet2D(train_data, i, 1, cvScalar(viz2));
        cvSet2D(train_data, i, 2, cvScalar( p));
        cvSet2D(train_data, i, 3, cvScalar(asp));
        cvSet2D(train_data, i, 4, cvScalar(media));
        cvSet2D(train_data, i, 5, cvScalar( dp));
        cvSet2D(train_data, i, 6, cvScalar(w));
        cvSet2D(train_data, i, 7, cvScalar(h));
    }
    knn = new CvKNearest(train_data, classes);
    fclose(f);
    return knn;
}

int classifica(CvKNearest *classificador, float *caracteristicas, int n)
{
    CvMat *samples;
    int i;
    float r;
    samples = cvCreateMat(1, n, CV_32FC1);
    for (i = 0; i < n; i++) {
        cvSet2D(samples, 0, i, cvScalar(caracteristicas[i]));
    }
    r = classificador->find_nearest(samples, 3);
    return (int) r;
}


float media_regiao(CvRect *r, IplImage *contrast_map)
{
    int i, j, n;
    float sum = 0.0;
    uchar val;
    n = r->width * r->height;
    for (i = 0; i < r->height; i++) {
        for (j = 0; j < r->width; j++) {
            val = contrast_map->imageData[(i + r->y) * contrast_map->widthStep + j + r->x];
            sum += val;
        }
    }
    return sum/n;
}

float dp_regiao(CvRect *r, IplImage *contrast_map, float media)
{
    int i, j, n;
    float sum = 0.0;
    uchar val;
    n = r->width * r->height;
    for (i = 0; i < r->height; i++) {
        for (j = 0; j < r->width; j++) {
            val = contrast_map->imageData[(i + r->y) * contrast_map->widthStep + j + r->x];
            sum += (val - media) * (val - media);
        }
    }
    return sqrt(sum/n);

}

void pre_processa_imagem_para_classificacao(IplImage *in, IplImage *out, int block_size, int delta)
{
    /* faz threshold adaptativo, uma abertura e acha os componentes conexos. */
//    cvAdaptiveThreshold(in, out, 255,
//        CV_ADAPTIVE_THRESH_GAUSSIAN_C, CV_THRESH_BINARY, PRODUTOS_BLOCK, PRODUTOS_DELTA);
    cvThreshold(in, out, 75, 255, CV_THRESH_BINARY);
    cvMorphologyEx(out, out, NULL,
        NULL, CV_MOP_CLOSE, 1);
}

float preenchimento_regiao(CvRect *r, IplImage *recorte)
{
    int i, j, n;
    float sum = 0.0;
    uchar val;
    n = r->width * r->height;
    for (i = 0; i < r->height; i++) {
        for (j = 0; j < r->width; j++) {
            val = recorte->imageData[(i + r->y) * recorte->widthStep + j + r->x];
            if (val == 0) sum++;
        }
    }
    return sum / n;
}

void extrai_caracteristicas(IplImage *contraste, IplImage *img, CvRect *r, float *caracteristicas)
{
    caracteristicas[VIZINHOS1] = caracteristicas[VIZINHOS2] = 0.0;
    caracteristicas[PREENCHIMENTO] = preenchimento_regiao(r, img);
    caracteristicas[ASPECTO] = 1.0 * r->width / r->height;
    caracteristicas[MEDIA_CONTRASTE] = media_regiao(r, contraste);
    caracteristicas[DP_CONTRASTE] = dp_regiao(r, contraste, caracteristicas[MEDIA_CONTRASTE]);
    caracteristicas[LARGURA] = 1.0 * r->width;
    caracteristicas[ALTURA] = 1.0 * r->height;
}
