#include <cv.h>
#include "classifier.hpp"
#include "definitions.h"
#include "contrast.h"

int separa_textos(IplImage *img, IplImage *mask, CvContour **words)
{
    int i;
    char cmd[256];
    CvContour *cc;
    cvMorphologyEx(mask, mask, NULL, 
        cvCreateStructuringElementEx(5, 1, 2, 0, CV_SHAPE_RECT, 0), CV_MOP_CLOSE, 5);
    cvDilate(mask, mask, NULL, 4);
    cvDilate(mask, mask, 
        cvCreateStructuringElementEx(11, 1, 5, 0, CV_SHAPE_RECT, 0), 2);
    cvAdaptiveThreshold(img, img, 255, 
        CV_ADAPTIVE_THRESH_GAUSSIAN_C, CV_THRESH_BINARY, 13, 25);
    cvSmooth(img, img, CV_GAUSSIAN, 5, 0,0,0);
    cc = connected_components(cvCloneImage(mask));
    *words = cc;
    for (i = 0; cc != NULL; cc = (CvContour *) cc->h_next, i++) {
        sprintf(cmd, "./palavras/word-%d.tif", i);
        salva_componente_conexo(img, mask, cc, cmd);
    }
    return i;
}

int passa_ocr(CvContour *palavra, char *text, int count)
{
    char cmd[1024];
    int tipo, j;
    FILE *ocr;
    sprintf(cmd, "tesseract palavras/word-%d.tif palavras/word-%d -l por 2> /dev/null", count, count);
    system(cmd);
    sprintf(cmd, "palavras/word-%d.txt", count);
    ocr = fopen(cmd, "r");
    for (j = 0; !feof(ocr); j++) {
        text[j] = fgetc(ocr);
    }
    if (j >= 3)
        text[j-3] = '\0';
    else 
        text[1] = '\0';
    fclose(ocr);
    tipo = nome_de_produto_ou_preco(text);
    printf("Posição: (%d, %d), OCR: %d %s\n", palavra->rect.x, palavra->rect.y, tipo, text);
    return tipo;
}

IplImage *extrai_mascara_de_textos(IplImage *img_color, IplImage *img_gray)
{
    CvKNearest *knn;
    CvContour *cc, *t, *palavras;
    IplImage *contraste, *mascara_textos;
    float caracteristicas[8];
    mascara_textos = cvCreateImage(cvGetSize(img_gray), IPL_DEPTH_8U, 1);
    cvZero(mascara_textos);
    printf("SSD");
    knn = treina_classificador("./treinamento_imagens/treinamento_ordenado.csv");
    processa_usando_piramide(cvCloneImage(img_color), &contraste, 2);
    pre_processa_imagem_para_classificacao(img_gray, img_gray, 7, 25);

    cc = (CvContour *) connected_components(cvCloneImage(img_gray));
    for (t = cc; t != NULL; t = (CvContour *) t->h_next) {
        extrai_caracteristicas(contraste, img_gray, (&t->rect), caracteristicas);
        if (classifica(knn, caracteristicas, N_FEAT) == CLASS_NOME) {
            cvRectangle(mascara_textos, cvPoint(t->rect.x, t->rect.y), 
                cvPoint(t->rect.width + t->rect.x, t->rect.height + t->rect.y), 
                CV_RGB(255, 255, 255), CV_FILLED, 8, 0);
        }
    }
    return mascara_textos;
}

int main(int argc, char *argv[])
{
    IplImage *img_color, *img_gray, *mascara_textos;
    CvContour *palavras; 
    int word_count, i, tipo;
    char text[4096];
    img_color = cvLoadImage(argv[1], 1);
    img_gray = cvLoadImage(argv[1], 0);
    mascara_textos = extrai_mascara_de_textos(img_color, cvCloneImage(img_gray));
    word_count = separa_textos(img_gray, mascara_textos, &palavras);
    printf("%d palavras encontradas.\n", word_count);
    for (i = 0; i < word_count; i++, palavras = (CvContour *) palavras->h_next) {
        tipo = LIXO; //passa_ocr(palavras, text, i);
        if (tipo == PRECO) {
            cvRectangle(img_color, cvPoint(palavras->rect.x, palavras->rect.y), 
                cvPoint(palavras->rect.width + palavras->rect.x - 1, palavras->rect.height + palavras->rect.y - 1), 
                CV_RGB(0, 0, 255), 3, 8, 0);
        } else if (tipo == NOME_PRODUTO) {
            cvRectangle(img_color, cvPoint(palavras->rect.x, palavras->rect.y), 
                cvPoint(palavras->rect.width + palavras->rect.x - 1, palavras->rect.height + palavras->rect.y - 1), 
                CV_RGB(0, 255, 0), 3, 8, 0);
        } else {
            cvRectangle(img_color, cvPoint(palavras->rect.x, palavras->rect.y), 
                cvPoint(palavras->rect.width + palavras->rect.x - 1, palavras->rect.height + palavras->rect.y - 1), 
                CV_RGB(255, 0, 0), 3, 8, 0);
        } 
    }    
    cvSaveImage("resultado-ocr.png", img_color, 0);
    return 0;
}
