#include "analisador.h"
#include "analisadorpda.h"
#include "definitions.h"

#include "util.h"

#include <cstdio>
#include <algorithm>
#include <cv.h>
#include <highgui.h>

#define CAMINHO_TESSERACT "/home/igor/Downloads/tesseract-3.00/api/"

int aceita_precos(int c)
{
    return c == ',' || isdigit(c);
}

void executa_tesseract(int min_size, int (*accept_func)(int))
{
    char texto[2048*10];
    char c;
    FILE *resultado_ocr;
    system(CAMINHO_TESSERACT "tesseract texto.tif resultado_ocr -l por 2> /dev/null");
    resultado_ocr = fopen("resultado_ocr.txt", "r");
    if (resultado_ocr == NULL) return;
    texto[0] = '\0';
    for (int j = 0; !feof(resultado_ocr);) {
        c = fgetc(resultado_ocr);
        if (c == '\n') c= ' ';
        if (!isalnum(c) && !isspace(c)) continue;
        if (!accept_func(c)) continue;
        texto[j] = c;
        texto[j+1] = '\0';
        j++;
    }
    fclose(resultado_ocr);
    system("rm resultado_ocr.txt");
    if (strlen(texto) >= min_size)
        printf("t: %s\n", texto);
}

int passa_ocr(IplImage *recorte, IplImage *recorte_mask, CvContour *produto, int min_size, int (*accept_func)(int))
{
    IplImage *recorte_gray = cvCreateImage(cvGetSize(recorte), IPL_DEPTH_8U, 1);
    cvCvtColor(recorte, recorte_gray, CV_BGR2GRAY);
    IplImage *r_copy = cvCloneImage(recorte_gray);

    cvAdaptiveThreshold(recorte_gray, recorte_gray, 255,
        CV_ADAPTIVE_THRESH_MEAN_C, CV_THRESH_BINARY, 13, 25);

    cvSmooth(recorte_gray, recorte_gray, CV_GAUSSIAN, 5, 0,0,0);
    cvAnd(recorte_mask, recorte_gray, recorte_gray, NULL);
    cvNot(recorte_mask, recorte_mask);
    cvOr(recorte_gray, recorte_mask, recorte_gray, NULL);
    cvNot(recorte_mask,recorte_mask);
    CvContour *textos = connected_components(recorte_mask);
    for(; textos != NULL; textos = (CvContour *) textos->h_next) {
        cvSetImageROI(recorte_gray, textos->rect);
        cvSetImageROI(r_copy, textos->rect);
        cvSaveImage("texto.tif", recorte_gray, 0);
        executa_tesseract(min_size, accept_func);
//        salva_com_nome_aleatorio(recorte_gray, "png");
//        salva_com_nome_aleatorio(r_copy, "png");
    }
    cvReleaseImage(&recorte_gray);
    return 0;
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        printf("Passe como argumento o nome da imagem a ser analisada\n");
        return -1;
    }
    IplImage *pagina = cvLoadImage(argv[1]);
    IplImage *pagina_textos = cvLoadImage(argv[1]);
    CvRect roi;        
    Analisador *pda = new AnalisadorPDA();
    IplImage *fundo = pda->segmenta_fundo(pagina);
    printf("Fundo segmentado\n");
    CvContour *produtos = pda->encontra_produtos(pagina, fundo);
    IplImage *mascara_produtos = cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 3);
    cvZero(mascara_produtos);
    printf("Produtos identificados\n");
    int i = 0;
    for (CvContour *t = produtos; t != NULL; t = (CvContour *) t->h_next, i++) {
        roi = pda->recorte_rect(pagina, t);
        cvRectangle(mascara_produtos, cvPoint(roi.x, roi.y), cvPoint(roi.x + roi.width, roi.y + roi.height),
            CV_RGB(255 & rand(),255 & rand(), 255 & rand()), CV_FILLED, 8, 0);
        cvDrawContours(mascara_produtos, (CvSeq *) t, CV_RGB(255, 255, 255), CV_RGB(255, 255, 255), -1, CV_FILLED, 8, cvPoint(0, 0));
        cvSetImageROI(pagina, roi);
        cvSetImageROI(fundo, roi);
//        blah37*5op_nome_aleatorio(pagina, "png");
        IplImage *recorte = cvCreateImage(cvSize(roi.width, roi.height), pagina->depth, pagina->nChannels);
        cvCopy(pagina, recorte, NULL);
        IplImage *recorte_fundo = cvCloneImage(fundo);
        IplImage *recorte_mask = pda->encontra_nomes_de_produtos(recorte, recorte_fundo, t);
//        printf("PRODUTO:");
//        passa_ocr(recorte, recorte_mask, t, 8, isprint);
        CvContour *cc = connected_components(recorte_mask);
        cvDrawContours(pagina_textos, (CvSeq *)cc, CV_RGB(255, 0, 0), CV_RGB(255, 0, 0), 3, 2, 8,cvPoint(roi.x, roi.y));
        recorte_mask = pda->encontra_precos(recorte, recorte_fundo, t);
//        printf("PREÇO:");
//        passa_ocr(recorte, recorte_mask, t, 2, aceita_precos);
        cc = connected_components(recorte_mask);
        cvDrawContours(pagina_textos, (CvSeq *) cc, CV_RGB(0, 0, 255), CV_RGB(0, 0, 255), 3, 2, 8,cvPoint(roi.x, roi.y));
        cvRectangle(pagina_textos, cvPoint(t->rect.x, t->rect.y), cvPoint(t->rect.x + t->rect.width, t->rect.y + t->rect.height),
            CV_RGB(0, 255, 0), 3, 8, 0);
//        salva_com_nome_aleatorio(recorte, "png");
        cvReleaseImage(&recorte);
        cvReleaseImage(&recorte_mask);
        cvReleaseImage(&recorte_fundo);
        printf("Produto %d analisado\n", i);
    }
    cvReleaseMemStorage(&produtos->storage);
    cvSetImageROI(pagina, cvRect(0, 0, pagina->width, pagina->height));
    cvSaveImage("textos.png", pagina_textos, 0);
    cvSaveImage("produtos.png", mascara_produtos, 0);
    return 0;
}
