#include "contrast.h"
#include "util.h"
#include "produto.h"
#include "classifier.hpp"

#include <cv.h>
#include <highgui.h>
#include <math.h>
#include <stdio.h>

IplImage *pega_linhas(IplImage *img_color)
{
    IplImage *vert, *hor, *linhas, *linhas3, *img, *img16b, *temp;
    int min_size = 40, line_thickness = 7;
    img = cvCreateImage(cvGetSize(img_color), IPL_DEPTH_8U, 1);
    img16b = cvCreateImage(cvGetSize(img_color), IPL_DEPTH_16S, 1);
    cvCvtColor(img_color, img, CV_RGB2GRAY);
    
    //cvMorphologyEx(img, img, temp, NULL, CV_MOP_GRADIENT, 3);
    cvNot(img, img);
    
    cvAdaptiveThreshold(img, img, 255, CV_ADAPTIVE_THRESH_GAUSSIAN_C, CV_THRESH_BINARY, 7, 10);
    cvSaveImage("th.png", img, 0);
    
    temp = cvCloneImage(img);
    vert = cvCloneImage(img);
    cvMorphologyEx(img, vert, NULL, 
        cvCreateStructuringElementEx(line_thickness, 1, line_thickness/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvMorphologyEx(vert, vert, NULL, 
        cvCreateStructuringElementEx(1, min_size, 0, min_size/2, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cvDilate(vert, vert, cvCreateStructuringElementEx(1, 2*min_size, 0, min_size, CV_SHAPE_RECT, NULL), 1);

    hor = cvCloneImage(img);
    cvMorphologyEx(img, hor, NULL, 
        cvCreateStructuringElementEx(1, line_thickness, 0, line_thickness/2, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvMorphologyEx(hor, hor, NULL, 
        cvCreateStructuringElementEx(min_size, 1, min_size/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cvDilate(hor, hor, cvCreateStructuringElementEx(2*min_size, 1, min_size, 0, CV_SHAPE_RECT, NULL), 1);
    
    linhas = cvCloneImage(img);
    cvOr(vert, hor, linhas, NULL);    
    cvDilate(linhas, linhas, NULL, 2);
    cvSaveImage("linhas.png", linhas, 0);
    linhas3 = cvCreateImage(cvGetSize(linhas), IPL_DEPTH_8U, 3);
    cvCvtColor(linhas, linhas3, CV_GRAY2RGB);
    cvReleaseImage(&img);
    return redimensiona_410x522(linhas3);
}

IplImage *segmenta_imagem(IplImage *linhas, CvContour *produtos) 
{
    CvContour *cc;
    int i, j;
    uchar val;
    IplImage *markers, *markers8bit, *edges;
    markers = cvCreateImage(cvGetSize(linhas), IPL_DEPTH_32S, 1);
    markers8bit = cvCreateImage(cvGetSize(linhas), IPL_DEPTH_8U, 1);
    for (cc = produtos, i = 1; cc != NULL; cc = cc->h_next, i++) {
        cvDrawContours(markers, cc, CV_RGB(i, i, i), CV_RGB(0,0,0), -1, CV_FILLED, 8, cvPoint(0,0)); 
    }
    cvWatershed(linhas, markers);
    cvConvertScale(markers, markers8bit, 255, 0);
    return markers8bit;
}

void salva_resultado(IplImage *original, IplImage *seg, CvContour *produtos)
{
    IplImage *seg3, *seg_redim;
    CvContour *cc;
    int i, x, y;
    seg3 = cvCloneImage(original);
    seg_redim = cvCreateImage(cvGetSize(original), original->depth, 1);
    cvResize(seg, seg_redim, CV_INTER_LINEAR);
    cvCvtColor(seg_redim, seg3, CV_GRAY2RGB);
    cvAddWeighted(original, 0.5, seg3, 0.5, 0, original);
    for (cc = produtos; cc != NULL; cc = cc->h_next) {
        y = cc->rect.y + cc->rect.height/2;
        x = cc->rect.x + cc->rect.width/2;
        cvCircle(original, cvPoint(x,y), 2, CV_RGB(0, 0, 0),5, 8, 0);
    }
    cvSaveImage("result.png", original, NULL);
}

void salva_regioes (IplImage *original, IplImage *seg_redim, CvKNearest *knn) 
{
    IplImage *mask, *masked;
    CvContour *regioes, *cc;
    char cmd[100];
    int i;
    Produto **produtos;
    cvNot(seg_redim, seg_redim);
    regioes = connected_components(seg_redim)->v_next;
    mask = cvCreateImage(cvGetSize(original), IPL_DEPTH_8U, 3);
    masked = cvCreateImage(cvGetSize(original), IPL_DEPTH_8U, 3);
    for (cc = regioes, i = 0; cc != NULL; cc = cc->h_next, i++);
    printf("%d produtos encontrados\n", i);
    produtos = malloc(sizeof(Produto *) * i);
    for (cc = regioes, i = 0; cc != NULL; cc = cc->h_next, i++) {
        printf("Processando produto %d\n", i);
        mascara_componente_conexo(original, cc, mask, masked);
        cvSetImageROI(masked, cc->rect);
        produtos[i] = new_Produto(masked);
        produto_pre_processa(produtos[i], knn);        
        sprintf(cmd, "c%d.png", i);
        cvSaveImage(cmd, produtos[i]->mascara_textos, 0);
        cvSetImageROI(masked, cvRect(0, 0, masked->width, masked->height));
    }
}

int main_app(int argc, char *argv[])
{
    IplImage *linhas, *seg, *seg_redim, *contraste;
    CvContour *produtos;
    IplImage *reduzida;
    CvKNearest *knn;
    IplImage *original = cvLoadImage(argv[1], 1);
    reduzida = redimensiona_410x522(original);
    produtos = processa_usando_piramide(cvCloneImage(reduzida), &contraste, 10);
    linhas = pega_linhas(original);
    
    seg = segmenta_imagem(linhas, produtos);
    seg_redim = cvCreateImage(cvGetSize(original), seg->depth, 1);
    cvResize(seg, seg_redim, CV_INTER_LINEAR);
    
    knn = treina_classificador("./treinamento_imagens/treinamento_ordenado.csv");
    
    salva_regioes(original, seg_redim, knn);
    
    //salva_resultado(reduzida, seg, produtos);
    cvReleaseImage(&reduzida);
    return 0;
}

