#include "definitions.h"
#include "produto.h"
#include <string.h>
#include <ctype.h>

Produto *new_Produto (IplImage *regiao)
{
    IplImage *temp;
    Produto *p = NULL;
    p = malloc(sizeof(Produto));
    temp = cvCloneImage(regiao);
    p->regiao = cvCreateImage(cvGetSize(temp), IPL_DEPTH_8U, 1);
    p->mascara_textos = cvCreateImage(cvGetSize(temp), IPL_DEPTH_8U, 1);
    cvCvtColor(temp, p->regiao, CV_RGB2GRAY);
    p->nome = NULL;
    p->preco = 0.0;
    return p;
}

int nome_de_produto_ou_preco (char *str)
{
    int i, len, num_letras, num_numeros, num_simbolos;
    len = strlen(str);
    num_letras = num_numeros = num_simbolos = 0;
    for (i = 0; i < len; i++) {
        num_letras += isalpha(str[i]);
        num_numeros += isdigit(str[i]);
        num_simbolos += isspace(str[i]) || ispunct(str[i]);
    }
    if (num_letras > num_numeros && num_letras > num_simbolos) 
        return NOME_PRODUTO;
    if (num_letras < num_numeros && num_numeros > num_simbolos) 
        return PRECO;
    return LIXO;
}


void produto_pre_processa(Produto *p, CvKNearest *knn)
{
    CvContour *cc, *t;
    int i;
    IplImage *contrast_map = NULL;
    char cmd[200];
    processa_usando_piramide(cvCloneImage(p->regiao), &contrast_map, 2); /* TODO: usar contraste na imagem colorida */
    pre_processa_imagem_para_classificacao(p->regiao, p->mascara_textos, 7, 25);
    cc = (CvContour *) connected_components(p->mascara_textos);
    p->mascara_textos = cvCloneImage(p->regiao);
    for (t = cc, i = 0; t != NULL; t = (CvContour *) t->h_next, i++) {
        extrai_caracteristicas(contrast_map, p->regiao, &(t->rect), p->caracteristicas);
        if (classifica(knn, p->caracteristicas, N_FEAT) == CLASS_NOME) {
            cvRectangle(p->mascara_textos, cvPoint(t->rect.x, t->rect.y), cvPoint(t->rect.width + t->rect.x, t->rect.height + t->rect.y), CV_RGB(127, 127, 127), 1, 8, 0);
        }
    }
    
}

void produto_passa_ocr(Produto *p)
{
    /* pega máscara de textos, recorta e passa o tesseract. */
}

void produto_imprime(Produto *p, FILE *f)
{
    fprintf(f, "Produto: %s\nPreco: %F\n", p->nome, p->preco);
}

void produto_deleta(Produto *p)
{
    free(p->nome);
    cvReleaseImage(&(p->regiao));
    free(p);
}

