#include "util.h"

double euclidean_distance(int r1, int b1, int g1, int r2, int b2, int g2)
{
    double d = 0.0;
    d = pow(r1-r2, 2) + pow(g1-g2, 2) + pow(b1-b2, 2);
    return sqrt(d);
}

double gaussian_distance(int r1, int b1, int g1, int r2, int b2, int g2, double sigma)
{
    double d = euclidean_distance(r1, b1, g1, r2, b2, g2);
    return (1.0 - exp(-d / (2 * SIGMA * SIGMA))) * 255;
}

int pixel_at(IplImage *img, int i, int j, int c)
{
    uchar val = img->imageData[img->widthStep * i + img->nChannels * j + c];
    return (int) val;
}

void set_pixel_at(IplImage *img, int i, int j, int c, int val) 
{
    uchar val_c;
    val_c = val;
    img->imageData[img->widthStep * i + img->nChannels * j + c] = val_c;
}

double width_average(CvContour *cc)
{
    double w = 0;
    int n = 0;
    while (cc != NULL) {
        n++;
        w += cc->rect.width;
        cc = (CvContour *) cc->h_next;
    }
    return w/n;
}

double height_average(CvContour *cc)
{
    double h = 0;
    int n = 0;
    while (cc != NULL) {
        n++;
        h += cc->rect.height;
        cc = (CvContour *) cc->h_next;
    }
    return h/n;
}

IplImage *redimensiona_410x522(IplImage *original)
{
    IplImage *reduzida = NULL;
    int width, height;
    CvSize reduz;
    double aspect_width;
    aspect_width = 1.0 * original->height / original->width;
    reduz = cvSize(410, 410 * aspect_width);
    reduzida = cvCreateImage(reduz, original->depth, original->nChannels);
    cvResize(original, reduzida, CV_INTER_LINEAR);
    return reduzida;
}

void pinta_retangulos(IplImage *img, CvContour *cc)
{
    CvPoint p1, p2;
    for (; cc != NULL; cc = cc->h_next) {
        p1.x = cc->rect.x; p1.y = cc->rect.y;
        p2.x = cc->rect.x + cc->rect.width; p2.y = cc->rect.y + cc->rect.height;
        cvRectangle(img, p1, p2, CV_RGB(255,0,0), 1, 8, 0);
    }
    cvSaveImage("seg.png", img, NULL);
}


CvContour *connected_components(IplImage *img)
{
    CvMemStorage *mem;
    IplImage *clone = cvCloneImage(img);
    CvSeq *contours = NULL;
    mem = cvCreateMemStorage(0);
    cvFindContours(clone, mem, &contours, sizeof(CvContour),
        CV_RETR_CCOMP,CV_CHAIN_APPROX_SIMPLE, cvPoint(0,0));
    cvReleaseImage(&clone);
    return (CvContour *) contours;
}

CvContour *pre_processa_cc(CvContour * cc, double wr, double hr, int cmp)
{
    double w_avg, h_avg;
    CvContour *ptr = cc;
    int cmp_val;
    w_avg = width_average(cc); h_avg = height_average(cc);
    while (ptr != NULL) {
        cmp_val = 0;
        if (cmp == SMALLER && (ptr->rect.width/w_avg >= wr || ptr->rect.height/h_avg >= hr)) {
            cmp_val = 1;
        } else if (cmp == BIGGER && (ptr->rect.width/w_avg <= wr || ptr->rect.height/h_avg <= hr)) {
            cmp_val = 1;
        }
        if (cmp_val) {
            if (ptr == cc) {
                cc = (CvContour *) cc->h_next;
                if (cc != NULL)
                    cc->h_prev = NULL;
                ptr = cc;
            } else {
                ptr->h_prev->h_next = ptr->h_next;
                if (ptr->h_next != NULL) {
                    ptr->h_next->h_prev = ptr->h_prev;
                }
                ptr = (CvContour *) ptr->h_next;
            }
        } else {
            ptr = (CvContour *) ptr->h_next;
        }
    }
    return cc;
}

void mascara_componente_conexo(IplImage *original, CvContour *cc, IplImage * mask, IplImage *masked) 
{
    cvZero(mask);
    cvDrawContours(mask, cc, CV_RGB(255, 255, 255), CV_RGB(0, 0, 0), 0, CV_FILLED, 8, cvPoint(0,0));
    cvNot(original, original);
    cvAnd(mask, original, masked, 0);
    cvNot(masked, masked);
    cvNot(original, original);
}

void salva_componente_conexo(IplImage *img, IplImage *mask, CvContour *cc, char *nome)
{
    cvSetImageROI(img, cc->rect);
    cvSetImageROI(mask, cc->rect);
    cvSaveImage(nome, img, mask);
    cvSetImageROI(img, cvRect(0, 0, img->width, img->height));
    cvSetImageROI(mask, cvRect(0, 0, mask->width, mask->height));
}

void salva_com_nome_aleatorio(IplImage *img, char *ext)
{
    char name[50];
    sprintf(name, "%d.%s", rand(), ext);
    cvSaveImage(name, img, 0);
}
